/*
 * Copyright (c) 1997 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <DemVtk/algebra3.h>
#include <DemVtk/demdataset.h>
#include <DemUnidraw/demcmds.h>
#include <DemUnidraw/demcomps.h>
#include <DemUnidraw/demeditor.h>
#include <DemUnidraw/lodactor.h>
#include <TopoFace/fgeomobjs.h>
#include <vtkMapper.h>
#include <vtkRenderer.h>
#include <vtkPolyDataMapper.h>
#include <InterViews/transformer.h>
#ifdef HAVE_ACE
#include <ace/SOCK_Stream.h>
#include <ace/SOCK_Connector.h>
#include <ace/INET_Addr.h>
#endif
#include <math.h>
#include <stdio.h>

float LODActor::fx[3];
float LODActor::fy[3];
int LODActor::usefootprint;

void LODActor::SetFootprint(float* x, float* y) {
  fx[0] = x[0];
  fx[1] = x[1];
  fx[2] = x[2];
  fy[0] = y[0];
  fy[1] = y[1];
  fy[2] = y[2];
}

void LODActor::UseFootprint(int usefp) {
  usefootprint = usefp;
}

LODActor::LODActor() : vtkOpenGLActor() {
  for (int i = 0; i < 6; i++)
    mappers[i] = NULL;
}

LODActor::~LODActor() {}

void LODActor::SetLODMapper(int level, vtkMapper* mapper) {
  if (level >= 0 && level <= 5)
    mappers[level] = mapper;
}

vtkMapper* LODActor::GetLODMapper(int level) {
  if (level >= 0 && level <= 5)
    return mappers[level];
  else
    return NULL;
}

void LODActor::Render(vtkRenderer* ren) {
  if (!this->Mapper) {
    int lod = 0;
    vtkCamera* cam = ren->GetActiveCamera();
    float cameraPos[3];
    cam->GetPosition(cameraPos);
    FPointObj cam_pos(cameraPos[0],cameraPos[1]);
    FBoxObj dsbox = DemEditor::GetBox(x, y);
    if (dsbox.Contains(cam_pos))
      lod = 0;
    else {
      FPointObj p1(dsbox._left, dsbox._bottom);
      FPointObj p2(dsbox._left, dsbox._top);
      FPointObj p3(dsbox._right, dsbox._top);
      FPointObj p4(dsbox._right, dsbox._bottom);
      float dist1 = cam_pos.Distance(p1);
      float dist2 = cam_pos.Distance(p2);
      float dist3 = cam_pos.Distance(p3);
      float dist4 = cam_pos.Distance(p4);
      float dist = FLT_MAX;
      dist = min(dist, dist1);
      dist = min(dist, dist2);
      dist = min(dist, dist3);
      dist = min(dist, dist4);
      if (dist < 2000.0)
	lod = 0;
      else if (dist < 3000.0)
	lod = 1;
      else if (dist < 4000.0)
	lod = 2;
      else
	lod = 3;
    }
    DemDataSet* demds = (DemDataSet*)DemEditor::GetDataSet(x, y, lod);
    vtkPolyDataMapper* mapper = vtkPolyDataMapper::New();
    WaterLandLookupTable* table = WaterLandLookupTable::New();
    mapper->SetLookupTable(table);
    mapper->SetInput(demds);
    SetMapper(mapper);
    GetProperty()->SetAmbient(0.15);
    GetProperty()->SetDiffuse(0.4);
    GetProperty()->SetAmbientColor(255.0/255.0, 218.0/255.0, 185.0/255.0);
  }
#if 0
  vtkActor::Render(ren);
}

void LODActor::RenderOld(vtkRenderer* ren) {
#endif
  if (!usefootprint)
    vtkActor::Render(ren);
  else {
  float bounds[6];
  vtkDataSet* dsb = /*mappers[5]*/GetMapper()->GetInput();
  dsb->GetBounds(bounds);
  FBoxObj dsbox(bounds[0], bounds[2], bounds[1], bounds[3]);
  float cx[3], cy[3];
  float cameraPos[3], cameraFP[3];
  vtkCamera* camera = ren->GetActiveCamera();
  camera->GetPosition((float *)cameraPos);
  camera->GetFocalPoint((float *)cameraFP);
#if 0
  cx[0] = cameraPos[0];
  cy[0] = cameraPos[1];
  vec2 camvec(cameraFP[0] - cameraPos[0], cameraFP[1] - cameraPos[1]);
  camvec.normalize();
  float range[2];
  camera->GetClippingRange(range);
  // camvec *= range[1];

  float anglev = camera->GetViewAngle();
  float angle3 = 90.0 - anglev/2.0;
  float lena = (range[1] / sin(angle3 * (M_PI / 180.0))); // times sin 90 = 1
  //  float lenc = (range[1] / sin(angle3 * (M_PI / 180.0))) 
  //    * sin((anglev/2.0)/(M_PI / 180.0));

  mat3 leftrot = rotation2D(vec2(0.,0.), angle3);
  vec2 leftvec = leftrot.inverse() * camvec;
  leftvec *= lena;
  mat3 riterot = rotation2D(vec2(0.,0.), -angle3);
  vec2 ritevec = riterot.inverse() * camvec;
  ritevec *= lena;
  cx[1] = cx[0] + leftvec[1];
  cy[1] = cy[0] + leftvec[0];
  cx[2] = cx[0] - ritevec[1];
  cy[2] = cy[0] - ritevec[0];
#endif
  FFillPolygonObj camview(fx, fy, 3);
#if 0
  FBoxObj cvbox;
  camview.GetBox(cvbox);
  char buf[256];
  sprintf(buf,
"mapfcl(\n gs(:fgcolor \"OrangeRed\",0,0,0 :brush 65535,0 :fillbg 0 :nonepat),\n\
area((%0.f,%0.f),(%0.f,%0.f),(%0.f,%0.f) :gs 0) :sw_e %0.f :sw_n %0.f)\n",
	  cx[0], cy[0], cx[1], cy[1], cx[2], cy[2], cvbox._left, cvbox._bottom);
  printf(buf);
  ACE_SOCK_Stream* _socket = new ACE_SOCK_Stream;
  ACE_SOCK_Connector* connector = new ACE_SOCK_Connector;
  ACE_INET_Addr* addr = new ACE_INET_Addr(20011, "localhost");

  if (connector->connect (*_socket, *addr) == -1) {
    ACE_ERROR ((LM_ERROR, "%p\n", "open"));
    return;
  }
  if (_socket->send (buf, strlen(buf)) == -1) {
    ACE_ERROR ((LM_ERROR, "%p\n", "send"));
    return;
  }
  //  _socket->close();
#endif
  if (camview.Intersects(dsbox)) {
#if 0
    float threshold = 40.0;
    int lod = 0;
    FPointObj cam_pos(cameraPos[0],cameraPos[1]);
    if (dsbox.Contains(cam_pos))
      lod = 0;
    else {
      float cx = dsbox._left + (dsbox._right - dsbox._left) / 2.0;
      float cy = dsbox._bottom + (dsbox._top - dsbox._bottom) / 2.0;
      FPointObj center(cx, cy);
      float dist = cam_pos.Distance(center);
      if (dist < 1000.0)
	lod = 0;
      else if (dist < 2000.0)
	lod = 1;
      else if (dist < 3000.0)
	lod = 2;
      else if (dist < 4000.0)
	lod = 3;
      else if (dist < 5000.0)
	lod = 4;
      else
	lod = 5;

      float* aspect;
      aspect = ren->GetAspect();
      for (lod = 0; lod < 6; lod++) {
	vtkDataSet* ds = mappers[lod]->GetInput();
	float p0[3], p1[3], p2[3], dp0[3], dp1[3], dp2[3];
	vtkCell* cell = ds->GetCell(0);
	cell->Points.GetPoint(0, p0);
	cell->Points.GetPoint(1, p1);
	cell->Points.GetPoint(2, p2);
	ren->SetWorldPoint(p0[0], p0[1], p0[2], 1.0);
	ren->WorldToDisplay();
	ren->GetDisplayPoint(dp0);
	ren->SetWorldPoint(p1[0], p1[1], p1[2], 1.0);
	ren->WorldToDisplay();
	ren->GetDisplayPoint(dp1);
	ren->SetWorldPoint(p2[0], p2[1], p2[2], 1.0);
	ren->WorldToDisplay();
	ren->GetDisplayPoint(dp2);
	float pdist1 = sqrt((dp1[1] - dp0[1]) * (dp1[1] - dp0[1]) +
			    (dp1[0] - dp0[0]) * (dp1[0] - dp0[0]));
	float pdist2 = sqrt((dp2[1] - dp1[1]) * (dp2[1] - dp1[1]) +
			    (dp2[0] - dp1[0]) * (dp2[0] - dp1[0]));
	float pdist3 = sqrt((dp0[1] - dp2[1]) * (dp0[1] - dp2[1]) +
			    (dp0[0] - dp2[0]) * (dp0[0] - dp2[0]));
	if (pdist1 < threshold && pdist2 < threshold && pdist3 < threshold)
	  continue;
	else
	  break;
      }
      if (lod == 6)
	lod = 5;
    }
    printf("actor lod: %d\n", lod);
    SetMapper(mappers[lod]);
#endif
    vtkActor::Render(ren);
  }
  //else
  //  printf("actor not in view\n");
  }
}
