/*
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <DlgUnidraw/dlgclasses.h>
#include <DlgUnidraw/dlgcmds.h>
#include <DlgUnidraw/dlgcomps.h>
#include <DlgUnidraw/dlgeditor.h>
#include <DlgUnidraw/dlgfunc.h>
#include <DlgUnidraw/dlgviewstate.h>
#include <MapServ/maphandler.h>
#include <MapUnidraw/mapcmds.h>
#include <MapUnidraw/mapstatecmds.h>
#include <MapUnidraw/mapstates.h>

#include <Dlg/dlgcov.h>
#include <Dlg/dlgdb.h>
#include <Dlg/dlglib.h>

#include <OverlayUnidraw/ovdoer.h>
#include <OverlayUnidraw/ovkit.h>

#include <UniIdraw/idvars.h>

#include <Unidraw/catalog.h>
#include <Unidraw/clipboard.h>
#include <Unidraw/ctrlinfo.h>
#include <Unidraw/iterator.h>
#include <Unidraw/kybd.h>
#include <Unidraw/keymap.h>
#include <Unidraw/selection.h>
#include <Unidraw/unidraw.h>
#include <Unidraw/viewer.h>
#include <Unidraw/Commands/edit.h>

#include <InterViews/border.h>
#include <InterViews/box.h>
#include <InterViews/frame.h>
#include <InterViews/glue.h>
#include <InterViews/menu.h>
#include <InterViews/message.h>
#include <InterViews/panner.h>
#include <InterViews/tray.h>
#include <InterViews/window.h>

#include <string.h>


/*****************************************************************************/

DlgEditor::DlgEditor(OverlayComp* gc, OverlayKit* ok) 
: MapEditor(false, ok)
{
    _projection = nil;
    _mapcomp = nil;

    Init(gc, "DlgEditor");

    Initializing(true);
}

DlgEditor::DlgEditor(boolean initflag, OverlayKit* ok) : MapEditor(initflag, ok) {
}

DlgEditor::DlgEditor(const char* file, OverlayKit* ok) : MapEditor(false, ok) {
  _projection = nil;
  _mapcomp = nil;
  if (file == nil) {
    Init(nil, "DlgEditor");
    Initializing(true);
  }
  else {
    Catalog* catalog = unidraw->GetCatalog();
    OverlayComp* comp;

    if (catalog->Retrieve(file, (Component*&) comp)) {
      Iterator it;
      comp->First(it);
      GraphicComp* dbcomp = comp->GetComp(it);
      if (dbcomp && dbcomp->IsA(MAPDB_COMP)) {
	DlgDatabase* db = (DlgDatabase*)((DlgDbComp*)dbcomp)->GetDatabase();
	SetMapDatabase(db);
	SetMapDbComp((DlgDbComp*)dbcomp);
      }
      Init(comp, "DlgEditor");
      Initializing(true);
    }
  }
}

void DlgEditor::Init (OverlayComp* comp, const char* name) {
  _curr_other = _prev_other = 0;
  _texteditor = nil;
  if (!comp) comp = new DlgIdrawComp;
  _overlay_kit->Init(comp, name);
  InitFrame();
}

void DlgEditor::OpenInit() {
    ((DlgKit*)_overlay_kit)->InitFClassState();
    MapEditor::OpenInit();
}

DlgDatabase* DlgEditor::GetMapDatabase() {
  return (DlgDatabase*)_mapdb;
}

void DlgEditor::SetMapDatabase(DlgDatabase* db) {
  MapEditor::SetMapDatabase(db);
}

void DlgEditor::InitCommands() {
  if (!_mapdb) {
    Catalog* catalog = unidraw->GetCatalog();
    char* mapdb_name = strdup(catalog->GetAttribute("database"));
    SetMapDatabase(new DlgDatabase(mapdb_name));
    Initializing(true);
    InitMapComp();
    MapShowCmd* show = new MapShowCmd(this);
    show->SetLibName(catalog->GetAttribute("library"));
    show->SetCovName(catalog->GetAttribute("coverage"));
    show->SetFClassName(catalog->GetAttribute("fclass"));
    show->SetUseEditor(false);
    CommandPusher* pusher = new CommandPusher(show);
    pusher->Push();
    _dbstate->notify();
    _libstate->notify();
    _covstate->notify();
    _fclassstate->notify();
  }
  else {
    MapShowAllCmd* showall = new MapShowAllCmd(this);
    Initializing(true);
    CommandPusher* pusher = new CommandPusher(showall);
    pusher->Push();
  }
  MapUpdateMenusCmd* umcmd = new MapUpdateMenusCmd(this);
  CommandPusher* pusher4 = new CommandPusher(umcmd);
  pusher4->Push();
}

void DlgEditor::InitMapComp() {
    _mapcomp = new DlgDbComp((DlgDatabase*)_mapdb);
    Clipboard* clb = new Clipboard();
    clb->Append(_mapcomp);
    PasteCmd* pcmd = new PasteCmd(this, clb);
    pcmd->Execute();
    Selection* s = GetViewer()->GetSelection();
    s->Clear();
}

void DlgEditor::FClassState(DlgFeatureClass* fc, boolean val) {
    ((DlgKit*)_overlay_kit)->FClassState(fc, val);
}

void DlgEditor::AddCommands(ComTerp* comterp) {

    comterp->add_command("load100kdlgfc", new Load100KDLGFCFunc(comterp));

    MapservHandler::AddCommands(comterp);
}
