/*
 * Copyright 1995 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */


#include <DlgUnidraw/dlgcomps.h>
#include <DlgUnidraw/dlgclasses.h>
#include <DlgUnidraw/dlgscripts.h>

#include <Dlg/dlgcov.h>
#include <Dlg/dlgdb.h>
#include <Dlg/dlgfclass.h>
#include <Dlg/dlglib.h>

#include <MapUnidraw/mapexport.h>

#include <OverlayUnidraw/ovclasses.h>
#include <OverlayUnidraw/paramlist.h>

#include <Unidraw/iterator.h>

#include <Attribute/attrlist.h>

#include <iostream.h>
#include <stdio.h>


/*****************************************************************************/

DlgFClassScript::DlgFClassScript (DlgFClassComp* subj) : MapFClassScript(subj) {}

ClassId DlgFClassScript::GetClassId () { return DLGFCLASS_SCRIPT; }

boolean DlgFClassScript::IsA (ClassId id) {
    return DLGFCLASS_SCRIPT == id || MapFClassScript::IsA(id);
}

int DlgFClassScript::ReadName (istream& in, void* addr1, void* addr2, void* addr3, void* addr4) {
    char name[BUFSIZ];
    ParamList::parse_string(in, name, BUFSIZ);

    if (!in.good()) {
	return -1;
    }
    else {
        *(DlgFeatureClass**)addr1 = new DlgFeatureClass(nil, name);
        return 0;
    }
}

boolean DlgFClassScript::Definition (ostream& out) {
  MapFClassComp* comp = (MapFClassComp*)GetSubject();
  MapFeatureClass* fclass = comp->GetFeatureClass();
  out <<  "mapfcl(\"" <<  fclass->name() << "\"" <<
    *comp->GetAttributeList() << ")";
}    

/*****************************************************************************/

DlgCovScript::DlgCovScript (DlgCovComp* subj) : MapCovScript(subj) {}

ClassId DlgCovScript::GetClassId () { return DLGCOV_SCRIPT; }

boolean DlgCovScript::IsA (ClassId id) {
    return DLGCOV_SCRIPT == id || MapCovScript::IsA(id);
}

int DlgCovScript::ReadName (istream& in, void* addr1, void* addr2, void* addr3, void* addr4) {
    char name[BUFSIZ];
    ParamList::parse_string(in, name, BUFSIZ);

    if (!in.good()) {
	return -1;
    }
    else {
        *(DlgCoverage**)addr1 = new DlgCoverage(nil, name);
        return 0;
    }
}

int DlgCovScript::ReadChildren (istream& in, void* addr1, void* addr2, void* addr3, void* addr4) {
    DlgFClassComp* child = nil;
    DlgCovComp* comp = (DlgCovComp*)addr1;
    char close_paren = ')';
    char key = ':';
    char lookahead;
    char buf[BUFSIZ];

    while (in.good()) {
	ParamList::skip_space(in);
	in.get(lookahead);

	/* keyword section */
	if (lookahead == ':' || lookahead == ')') {
            in.putback(lookahead);
	    break;
	}
	if (lookahead != ',')
            in.putback(lookahead);          
	else
	    ParamList::skip_space(in);
	ParamList::parse_token(in, buf, BUFSIZ);

	if (strcmp(buf,      "mapfcl") == 0)      
	    child = new DlgFClassComp(in);

	else {
	    fprintf(stderr, "unknown fclass object %s, skipping\n", buf);
	    continue;
	}

	if (child != nil) {
	    if (in.good() && child->valid()) {
		comp->Append(child);
	    } else {
	        /* report failure even if one failure */
		delete child;
		return -1;
	    }
	}
    }
    return 0;
}

/*****************************************************************************/

DlgLibScript::DlgLibScript (DlgLibComp* subj) : MapLibScript(subj) {}

ClassId DlgLibScript::GetClassId () { return DLGLIB_SCRIPT; }

boolean DlgLibScript::IsA (ClassId id) {
    return DLGLIB_SCRIPT == id || MapLibScript::IsA(id);
}

int DlgLibScript::ReadName (istream& in, void* addr1, void* addr2, void* addr3, void* addr4) {
    char name[BUFSIZ];
    ParamList::parse_string(in, name, BUFSIZ);

    if (!in.good()) {
	return -1;
    }
    else {
        *(DlgLibrary**)addr1 = new DlgLibrary(nil, name);
        return 0;
    }
}

int DlgLibScript::ReadChildren (istream& in, void* addr1, void* addr2, void* addr3, void* addr4) {
    DlgCovComp* child = nil;
    DlgLibComp* comp = (DlgLibComp*)addr1;
    char close_paren = ')';
    char key = ':';
    char lookahead;
    char buf[BUFSIZ];

    while (in.good()) {
	ParamList::skip_space(in);
	in.get(lookahead);

	/* keyword section */
	if (lookahead == ':' || lookahead == ')') {
            in.putback(lookahead);
	    break;
	}
	if (lookahead != ',')
            in.putback(lookahead);          
	else
	    ParamList::skip_space(in);
	ParamList::parse_token(in, buf, BUFSIZ);

	if (strcmp(buf,      "mapcov") == 0)      
	    child = new DlgCovComp(in);

	else {
	    fprintf(stderr, "unknown coverage object %s, skipping\n", buf);
	    continue;
	}

	if (child != nil) {
	    if (in.good() && child->valid()) {
		comp->Append(child);
	    } else {
	        /* report failure even if one failure */
		delete child;
		return -1;
	    }
	}
    }
    return 0;
}

/*****************************************************************************/

DlgDbScript::DlgDbScript (DlgDbComp* subj) : MapDbScript(subj) {}

ClassId DlgDbScript::GetClassId () { return DLGDB_SCRIPT; }

boolean DlgDbScript::IsA (ClassId id) {
    return DLGDB_SCRIPT == id || MapDbScript::IsA(id);
}

int DlgDbScript::ReadPath (istream& in, void* addr1, void* addr2, void* addr3, void* addr4) {
    char path[BUFSIZ];
    ParamList::parse_string(in, path, BUFSIZ);

    if (!in.good()) {
	return -1;
    }
    else {
        *(DlgDatabase**)addr1 = new DlgDatabase(path);
        return 0;
    }
}

int DlgDbScript::ReadChildren (istream& in, void* addr1, void* addr2, void* addr3, void* addr4) {
    DlgLibComp* child = nil;
    DlgDbComp* comp = (DlgDbComp*)addr1;
    char close_paren = ')';
    char key = ':';
    char lookahead;
    char buf[BUFSIZ];

    while (in.good()) {
	ParamList::skip_space(in);
	in.get(lookahead);

	/* keyword section */
	if (lookahead == ':' || lookahead == ')') {
            in.putback(lookahead);
	    break;
	}
	if (lookahead != ',')
            in.putback(lookahead);          
	else
	    ParamList::skip_space(in);
	ParamList::parse_token(in, buf, BUFSIZ);

	if (strcmp(buf,      "maplib") == 0)      
	    child = new DlgLibComp(in);

	else {
	    fprintf(stderr, "unknown library object %s, skipping\n", buf);
	    continue;
	}

	if (child != nil) {
	    if (in.good() && child->valid()) {
		comp->Append(child);
	    } else {
	        /* report failure even if one failure */
		delete child;
		return -1;
	    }
	}
    }
    return 0;
}

/*****************************************************************************/

DlgIdrawScript::DlgIdrawScript (DlgIdrawComp* subj) : MapIdrawScript(subj) {}

ClassId DlgIdrawScript::GetClassId () { return DLG_IDRAW_SCRIPT; }

boolean DlgIdrawScript::IsA (ClassId id) { 
    return DLG_IDRAW_SCRIPT == id || MapIdrawScript::IsA(id);
}

boolean DlgIdrawScript::Emit (ostream& out) {
    Command *cmd = GetCommand();
    ExportScriptType exporttype;
    if (cmd != nil && cmd->IsA(MAPEXPORT_CMD))
        exporttype = ((MapExportCmd*)cmd)->type();
    else
        exporttype = DlgIdrawType;

    if (exporttype == DlgIdrawType)
        out << "dlgviewer(";
    if (exporttype == MapIdrawType)
        out << "mapdraw(";
    if (exporttype == OvIdrawType)
        out << "drawtool(";

    Iterator i;
    boolean status = true;
    First(i);
    if (!Done(i)) out << "\n";
    for (; status && !Done(i); ) {
	ExternView* ev = GetView(i);
        if (exporttype != OvIdrawType)	
            Indent(out);
        status = ev->Definition(out);
	Next(i);
	if (!Done(i)) out << ",\n";
    }

    out << "\n";
    FullGS(out);
    Annotation(out);
    out << ")\n";
    return status;
}

int DlgIdrawScript::ReadChildren (istream& in, void* addr1, void* addr2, void* addr3, void* addr4) {
    DlgDbComp* child = nil;
    DlgIdrawComp* comp = (DlgIdrawComp*)addr1;
    char close_paren = ')';
    char key = ':';
    char lookahead;
    char buf[BUFSIZ];

    while (in.good()) {
	ParamList::skip_space(in);
	in.get(lookahead);

	/* keyword section */
	if (lookahead == ':' || lookahead == ')') {
            in.putback(lookahead);
	    break;
	}
	if (lookahead != ',')
            in.putback(lookahead);          
	else
	    ParamList::skip_space(in);
	ParamList::parse_token(in, buf, BUFSIZ);

	if (strcmp(buf,      "mapdb") == 0)      
	    child = new DlgDbComp(in);

	else {
	    fprintf(stderr, "unknown database object %s, skipping\n", buf);
	    continue;
	}

	if (child != nil) {
	    if (in.good() && child->valid()) {
		comp->Append(child);
	    } else {
	        /* report failure even if one failure */
		delete child;
		return -1;
	    }
	}
    }
    return 0;
}
