/*
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

/* map coverage object */

#ifndef mapcov_h
#define mapcov_h

#include <Map/mapfiles.h>

class MapEdge;
class MapFace;
class MapFeatureClass;
class MapLibrary;
class MapNode;
class ostream;

class MapCoverage : public MapFiles {
protected:
    MapCoverage (MapLibrary*, const char* name);

public:
    virtual ~MapCoverage ();

    virtual const char* path() const;
    MapLibrary* library() const { return _library; }
    virtual void library(MapLibrary* lib) { _library = lib; }

    int nfcls() const;
    MapFeatureClass* feature_class(unsigned fclsnum) const;
    MapFeatureClass* feature_class(const char*) const;

    int nedges() const;
    MapEdge* edge(unsigned edgenum) const;

    int nfaces() const;
    MapFace* face(unsigned facenum) const;

    int nnodes() const;
    MapNode* node(unsigned nodenum) const;

    virtual const char* description() const;
    friend ostream& operator << (ostream& s, const MapCoverage&);

protected:
    boolean initialized() const { return _nfcls>=0; }
    void init_feature_classes();

    virtual char** feature_class_names(int &ncovs) const = 0;
    virtual MapFeatureClass* make_feature_class(const char* name) = 0;

    void init_primitive_tables(int nedges, int nfaces, int nnodes);

protected:
    MapLibrary* _library;

    MapFeatureClass** _fclasses;
    int _nfcls;

    MapEdge** _edges;
    MapFace** _faces;
    MapNode** _nodes;
    int _nedges;
    int _nfaces;
    int _nnodes;

};

#endif /* defined(mapcov_h) */
